module.exports = function(grunt) {

	var contentlocker = {
		banner: '/*!\n' +
		 '* Content Locker\n' +
		 '*\n' +
		 '* (c) Copyright mythemeshop.com\n' +
		 '*\n' +
		 '* @version 1.0.0\n' +
		 '* @author  MyThemeShop\n' +
		 '*/\n',
		 uglify : {
 			core: {
 				'assets/js/cl-front.js' : [

					// Base
					'assets/src/init.js',
					'assets/src/settings.js',
					'assets/src/lang.js',
					'assets/src/helpers.js',
					'assets/src/eventmanager.js',

					// SDKs
					'assets/src/sdks.js',
					'assets/src/sdks/facebook.js',
					'assets/src/sdks/twitter.js',
					'assets/src/sdks/google.js',
					'assets/src/sdks/linkedin.js',
					'assets/src/sdks/email.js',

					// Social Controls
					'assets/src/controls.js',
					'assets/src/controls/facebook-like.js',
					'assets/src/controls/twitter-tweet.js',
					'assets/src/controls/google-plus.js',

					// Signin Controls
					'assets/src/controls-signin.js',
					'assets/src/controls/facebook-signin.js',
					'assets/src/controls/google-signin.js',
					'assets/src/controls/linkedin-signin.js',
					'assets/src/controls/twitter-signin.js',
					'assets/src/controls/email-signin.js',

					// Locker
					'assets/src/extras.js',
					'assets/src/locker-state.js',
					'assets/src/locker.js',
					'assets/src/social-locker.js',
					'assets/src/signin-locker.js',


					// Always in the last
					'assets/src/manager.js'
 				]
 			}
 		}
	};

    // Project configuration.
	grunt.initConfig({
		pkg: grunt.file.readJSON('package.json'),

        uglify: {
			core: {
				options: {
					banner : contentlocker.banner + '\n\n' + '(function ($) {' + '\n\n',
					footer : '\n\n' + '})(jQuery);',
					preserveComments: 'some',
					beautify: false,
					compress: true,
					quoteStyle: 3,
					mangle  : false
				},
                files: contentlocker.uglify.core
            }
        },

		// Generate POT files.
		makepot: {
			dist: {
				options: {
					type: 'wp-plugin',
					mainFile: 'content-locker.php',
					domainPath: 'languages',
					potHeaders: {
						'report-msgid-bugs-to': 'https://community.mythemeshop.com/',
						'language-team': 'MyThemeShop <support-team@mythemeshop.com>'
					},
					potFilename: 'content-locker.pot',
					exclude: [
						'node_modules/.*'
					]
				}
			}
		},

		// Check textdomain errors.
        checktextdomain: {
            options: {
                keywords: [
                    '__:1,2d',
                    '_e:1,2d',
                    '_x:1,2c,3d',
                    'esc_html__:1,2d',
                    'esc_html_e:1,2d',
                    'esc_html_x:1,2c,3d',
                    'esc_attr__:1,2d',
                    'esc_attr_e:1,2d',
                    'esc_attr_x:1,2c,3d',
                    '_ex:1,2c,3d',
                    '_n:1,2,4d',
                    '_nx:1,2,4c,5d',
                    '_n_noop:1,2,3d',
                    '_nx_noop:1,2,3c,4d'
                ]
            },

			plugin: {
                src: ['**/*.php', '!node_modules/**'],
                expand: true,
                options: {
                    text_domain: ['content-locker', 'cmb2']
                }
            }
        }
	});

	// Default task(s).
    grunt.registerTask('default', [
		'uglify'
    ]);

	grunt.registerTask( 'pot', [
		'checktextdomain', 'makepot'
    ]);

    // Load the plugin that provides the tasks.
    grunt.loadNpmTasks('grunt-contrib-uglify');
	grunt.loadNpmTasks( 'grunt-wp-i18n' );

	// Quality Assurance
    grunt.loadNpmTasks( 'grunt-checktextdomain' );
};
