/*!
 * Control Base
 *
 * @since 1.0.0
 * @pacakge core
*/
var ControlSigninBase = ControlBase.extend({

	render: function( container ) {

		this.container = container;
		this.el = container.find( '.mts-cl-connect-' + this.name );

		// Checks
		if ( this.has_error() ) {
			this.show_error();
			return this;
		}

		var self = this, obj;

		self.set_loading_state();
		obj = cl.sdk.connect( self.sdk );

		obj.fail(function( result ) {
			self.remove_loading_state();
			self.show_log( result );
		});

		obj.done(function() {
			self._create_events();
			self.create_events();
			self._render();
			self.handle_click();
		});

		return this;
	},

	handle_click: function() {

		var self = this;

		self.el.click(function() {
			self.connect(function( data, serviceData ) {
				self.run_actions( data, serviceData, true );
			});

			return false;
		});
	},

	get_identity: function() {

		if ( this._identity ) {
			return this._identity;
		}

		this._identity = 'mts_content_locker_' + cl.helpers.hash( this.name ) + '_hash_' + this.name;

		return this._identity;
	},

	connect: function() {
		throw new Error( 'The control should implement the method `connect`' );
	},

	run_actions: function( data, serviceData, loader ) {

		var self = this,
			def = $.Deferred();

		if ( ! self._actions_done ) {
			self._actions_done = true;

			var actions = self.options.actions.slice();

			if ( loader ) {
				self.locker.screen( 'loader' );
			}

			var run = function() {
				var action = actions.shift();

				if ( ! action ) {
					def.resolve();
					cl.active_locker.unlock( 'button', self.name, self );
					return;
				}

				var func = 'run_' + action + '_action';

				if ( ! self[func] ) {
					def.reject();
					self._actions_done = false;
					throw new Error( 'The action \'' + func + '\' not found.' );
				}

				self[func]( data, serviceData, action, loader, function( status ) {

					if ( 'error' === status ) {
						self.locker.do_action( 'raw-error' );
						def.reject( status );
						self._actions_done = false;
						self.locker.screen( 'default' );
						return;
					}

					run();
				});
			};

			run();
			return def.promise();
		}
	},

	run_signup_action: function( lead, serviceData, action, loader, callback ) {

		var self = this;
		var run = function() {

			if ( loader ) {
				self.locker.screen( 'loader' );
			}

			var data = {};
			data.identity = lead;
			data.handler = 'signup';
			data.context = self.locker.get_context_data();

			// For developers
			data = cl.hooks.apply_filters( self.locker.id + '.ajax-data', data );
			data = cl.hooks.apply_filters( self.locker.id + '.lead.ajax-data', data );

			// Prepare for submit
			data.action = 'mts_cl_connect';
			data.security = __contentlocker.security;

			$.ajax({
				url: __contentlocker.ajaxurl,
				type: 'POST',
				dataType: 'json',
				data: data,

				success: function( response ) {

					if ( response && response.success ) {
						callback();
					}

					if ( response && ! response.success ) {
						self.show_notice( response.error );
						callback( 'error' );
					}
				},

				error: function( response ) {

					if ( response && response.readyState < 4 ) {
						self.locker.screen( 'default' );
						self.show_notice( 'Unable to sign in, the ajax error occured.' );
						callback( 'error' );
						console.log( 'Invalide ajax response: ' + response.responseText );
					}
				}
			});
		};

		// If has email
		if ( lead.email ) {
			run();
			return;
		}

		// If no email
		self.locker.screen( 'enter-email', {
			title: cl.__.onestep.screen_title,
			message: cl.__.onestep.screen_instruction,
			button_title: cl.__.onestep.screen_button,
			note: cl.__.notices.no_spam,
			callback: function( email ) {
				lead.email = email;
				run();
			}
		});
	},

	run_lead_action: function( lead, serviceData, action, loader, callback ) {

		var self = this;
		var run = function() {

			if ( loader ) {
				self.locker.screen( 'loader' );
			}

			var data = {};
			lead.list_id = self.options.list_id;
			data.identity = lead;
			data.handler = 'lead';
			data.context = self.locker.get_context_data();

			// For developers
			data = cl.hooks.apply_filters( self.locker.id + '.ajax-data', data );
			data = cl.hooks.apply_filters( self.locker.id + '.lead.ajax-data', data );

			// Prepare for submit
			data.action = 'mts_cl_connect';
			data.security = __contentlocker.security;

			$.ajax({
				url: __contentlocker.ajaxurl,
				type: 'POST',
				dataType: 'json',
				data: data,

				success: function( response ) {

					if ( response && response.success ) {
						callback();
					}

					if ( response && ! response.success ) {
						self.show_notice( response.error );
						callback( 'error' );
					}
				},

				error: function( response ) {

					if ( response && response.readyState < 4 ) {
						self.locker.screen( 'default' );
						self.show_notice( 'Unable to sign in, the ajax error occured.' );
						callback( 'error' );
						console.log( 'Invalide ajax response: ' + response.responseText );
					}
				}
			});
		};

		// If has email
		if ( lead.email ) {
			run();
			return;
		}

		// If no email
		self.locker.screen( 'enter-email', {
			title: cl.__.onestep.screen_title,
			message: cl.__.onestep.screen_instruction,
			button_title: cl.__.onestep.screen_button,
			note: cl.__.notices.no_spam,
			callback: function( email ) {
				lead.email = email;
				run();
			}
		});
	},

	// Private
	_track_window: function( search, callback ) {

		var original = window.open;

		window.open = function( url, name, specs, replace ) {
			var popup = original( url, name, specs, replace );

			if ( ! popup ) {
				return popup;
			}

			if ( ! url.includes( search ) ) {
				return popup;
			}

			var interval = setInterval( function() {

				if ( popup && popup.closed ) {
					clearInterval( interval );

					if ( callback ) {
						callback();
					}
				}
			}, 300 );

			return popup;
		};
	}
});
cl.controls.base_signin = ControlSigninBase;
