/*!
 * Control Base
 *
 * @since 1.0.0
 * @pacakge core
*/
cl.controls = {};

var ControlBase = function( options, locker ) {

	this.errors = [];
	this.is_loading = false;
	this.el = null;
	this.container = null;

	this.locker = locker;
	this.options = $.extend( true, {}, this.defaults, options );
	this.verification = $.extend( true, {}, { container: 'iframe', timeout: 5e3 }, this.verification );

	if ( ! this.name ) {
		throw new Error( 'The property "name" cannot be empty for the control.' );
	}

	this.init.apply( this, arguments );
};
ControlBase.extend = cl.helpers.extend;

_.extend( ControlBase.prototype, {

	init: function() {},

	render: function( container ) {

		this.container = container;
		this.el = container.find( '.mts-cl-control-placeholder' );

		// Check
		if ( this.has_error() ) {
			this.show_error();
			return this;
		}

		var self = this, obj;

		self.set_loading_state();
		obj = cl.sdk.connect( self.sdk );

		obj.fail(function( result ) {
			self.remove_loading_state();
			self.show_log( result );
		});

		obj.done(function() {
			self._create_events();
			self.create_events();
			self._render();
			var def = self.verify_button();

			def.always(function() {
				self.remove_loading_state();
			});

			def.fail(function( result ) {
				self.show_log( result );
			});
		});

		return this;
	},

	_create_events: function() {

		var self = this;

		this.container.on( 'mouseenter', function() {

			cl.active_locker = self.locker;
		});
	},

	get_identity: function() {

		if ( this._identity ) {
			return this._identity;
		}

		this._identity = 'page_' + cl.helpers.hash( this.options.url ) + '_hash_' + this.name;

		return this._identity;
	},

	create_events: function() {

	},

	unlock: function( type, name, control ) {
		var self = this;
		self.locker.unlock( type || 'button', name || self.name, control || self );
	},

	verify_button: function() {
		var self = this,
			def = $.Deferred(),
			timer = self.verification.timeout;

		var verify = function() {

			if ( 0 === self.container.find( self.verification.container ).length && timer >= 0 ) {
				setTimeout(function() {
					verify();
				}, 500 );

				timer -= 500;

				return;
			}

			if ( 0 >= timer ) {
				var notice = cl.__.notices.unable_create_control.replace( '{0}', self.sdk );
				def.reject( notice );
				return;
			}

			def.resolve();
		};

		verify();

		return def.promise();
	},

	// Loader
	set_loading_state: function() {
		this.is_loading = true;
		this.container.addClass( 'mts-cl-state-loading' );
	},

	remove_loading_state: function() {
		this.is_loading = false;
		this.container.removeClass( 'mts-cl-state-loading' );
	},

	// Error & Log
	show_log: function( result ) {
		console.log( result );
	},

	show_notice: function( msg ) {
		var self = this,
			notice = $( '<div class="mts-cl-group-notice">' + msg + '</div>' ),
			group = self.container.parent();

		notice.hide();
		group.find( '.mts-cl-group-error' ).remove();
		group.append( notice );
		notice.fadeIn( 500 );
		setTimeout(function() {
			notice.fadeOut( 500, function() {
				notice.remove();
			});
		}, 5000 );
	},

	show_error: function() {
		var self = this;

		if ( ! self.container.hasClass( 'mts-cl-state-error' ) ) {
			var notice = $( '<div class="mts-cl-error-body"><a href="#" class="mts-cl-error-title">' + cl.__.error + '</a></div>' );

			notice.click(function() {

				self.show_notice( self.errors.join( '<br>' ) );

				return false;
			});

			self.container.removeClass( 'mts-cl-state-loading' ).addClass( 'mts-cl-state-error' );
			self.el.replaceWith( notice );
		}
	},

	set_error: function( msg ) {
		this.errors.push( msg );
	},

	has_error: function() {
		return this.errors.length ? true : false;
	}
});
cl.controls.base = ControlBase;
