/*!
 * Linkedin Signin Control
 *
 * @since 1.0.0
 * @pacakge core
*/
var LinkedinSignin = ControlSigninBase.extend({

	sdk: 'linkedin-connect',
	name: 'linkedin',

	defaults: {},

	init: function() {

		if ( ! cl.settings.get( 'linkedin' ).client_id ) {
			this.set_error( cl.__.notices.empty_linkedin_clientid );
		}
	},

	_render: function() {

		this.remove_loading_state();
	},

	connect: function( callback ) {

		var self = this,
			result = false;

		self._track_window( 'platform.linkedin.com', function() {
			setTimeout(function() {

				if ( result ) {
					return;
				}

				self.locker.do_action( 'raw-social-app-declined' );
				self.show_notice( cl.__.errors.not_signed_in );

			}, 500 );
		});

		IN.User.authorize(function( response ) {

			result = true;
			self._identify(function( status, data ) {

				if ( 'error' === status ) {
					self.show_notice( cl.__.notices.no_user_data + ': ' + data );
					return;
				}

				callback( data, response );

			});
		});
	},

	_identify: function( callback ) {

		var fields = [ 'firstName', 'lastName', 'emailAddress', 'publicProfileUrl', 'pictureUrls::(original)' ];
		IN.API.Profile( 'me' ).fields( fields.join( ',' ) ).result( function( response ) {

			var data = {},
				me = response.values[0];

			data.source = 'linkedin';
			data.social = true;
			data.email = me.emailAddress;
			data.display_name = me.firstName + ' ' + me.lastName;
			data.name = me.firstName;
			data.family = me.lastName;
			data.linkedin_url = me.publicProfileUrl;
			data.image = (function() {
				if ( me.pictureUrls && me.pictureUrls.values ) {
					return me.pictureUrls.values[0];
				}

				return '';
			})();

			callback( 'success', data );

		}).error(function( response ) {
			callback( 'error', response );
		});
	}
});
cl.controls.linkedin = LinkedinSignin;
