/*!
 * Twitter Signin Control
 *
 * @since 1.0.0
 * @pacakge core
*/
var TwitterSignin = ControlSigninBase.extend({

	sdk: 'twitter',
	name: 'twitter',

	defaults: {},

	_render: function() {

		this.remove_loading_state();
	},

	connect: function( callback ) {

		var self = this;

		if ( cl.twitterOAuthReady ) {
			self._identify( function( data ) {
				callback( data, { visitor_id: self._visitor_id } );
			});
		} else {
			var options = {
					handler: 'twitter',
					request: 'init',
					options: JSON.stringify( self.options ),
					action: 'mts_cl_connect',
					security: __contentlocker.security
				},
				twid = cl.helpers.store.cookie( 'mts_cl_twid' );

			if ( twid && 'null' !== twid ) {
				options.visitor_id = twid;
			}

			var url = __contentlocker.ajaxurl + '?' + $.param( options );

			self._track_window( 'handler=twitter', function() {
				setTimeout(function() {

					if ( cl.twitterOAuthReady ) {
						return;
					}

					self.locker.do_action( 'raw-social-app-declined' );
					self.show_notice( cl.__.errors.not_signed_in );

				}, 500 );
			});

			window.open( url, 'Twitter Sign-In', 'width=500,height=450,resizable=yes,scrollbars=yes,status=yes' );

			window.mts_cl_twitter_oauth_completed = function( res ) {
				cl.twitterOAuthReady = true;
				self._set_visitor_id( res );
				self.connect( callback );
			};

			window.mts_cl_twitter_oauth_denied = function( res ) {
				self.locker.do_action( 'raw-social-app-declined' );
				self.show_notice( cl.__.errors.not_signed_in );
				self._set_visitor_id( res );
			};
		}
	},

	_set_visitor_id: function( id ) {
		this._visitor_id = id;
		cl.helpers.store.cookie( 'mts_cl_twid', id, {
			expires: '2s'
		});
	},

	_identify: function( callback ) {

		var self = this;

		var request = $.ajax({
			url: __contentlocker.ajaxurl,
			type: 'POST',
			dataType: 'json',
			data: {
				handler: 'twitter',
				request: 'user_info',
				visitor_id: self._visitor_id,
				action: 'mts_cl_connect',
				security: __contentlocker.security
			},

			success: function( response ) {

				var data = {};

				if ( ! response || response.error || response.errors ) {
					callback( data );
					console.log( 'Unable to get the user data:' + request.responseText );
					return;
				}

				data.source = 'twitter';
				data.social = true;
				data.email = response.email;
				data.display_name = response.screen_name;

				if ( response.name ) {
					var name = response.name.split( ' ', 2 );
					if ( 2 === name.length ) {
						data.name = name[0];
						data.family = name[1];
					} else {
						data.name = response.name;
					}
				} else {
					data.name = response.name;
				}
				data.twitter_url = 'https://twitter.com/' + response.screen_name;
				data.image = (function() {
					if ( response.profile_image_url  ) {
						return response.profile_image_url.replace( '_normal', '' );
					}

					return '';
				})();

				callback( data );
			},

			error: function( response ) {

				console.log( 'Unable to get the user data: ' + response.responseText );
				callback({});
			}
		});
	},

	run_follow_action: function( lead, service, action, loader, callback ) {

		var self = this;

		var data = {};
		data.handler = 'twitter';
		data.request = 'follow';
		data.visitor_id = self._visitor_id;
		data.follow_to = self.options.follow.user;
		data.notification = self.options.follow.notification;
		data.context = self.locker.get_context_data();

		// For developers
		data = cl.hooks.apply_filters( self.locker.id + '.ajax-data', data );
		data = cl.hooks.apply_filters( self.locker.id + '.twitter-follow.ajax-data', data );

		// Prepare for submit
		data.action = 'mts_cl_connect';
		data.security = __contentlocker.security;

		var request = $.ajax({
			url: __contentlocker.ajaxurl,
			type: 'POST',
			dataType: 'json',
			data: data,

			success: function( response ) {

				if ( ! response || response.error || response.errors ) {
					console.log( 'Unable to follow:' + request.responseText );
				}

				if ( response && response.error ) {
					self.locker.screen( 'default' );
					self.show_notice( 'Unable to perform the follow action due to the error: ' + response.error );
					return;
				} else {
					callback();
				}
			},

			error: function() {

				self.locker.screen( 'default' );
				self.show_notice( 'Unable to perform the follow action due to the unexpected error. See the logs for more details.' );
				console.log( 'Unable to follow: ' + request.responseText );
			}
		});
	},

	run_tweet_action: function( lead, service, action, loader, callback ) {

		var self = this;

		var data = {};
		data.handler = 'twitter';
		data.request = 'tweet';
		data.visitor_id = self._visitor_id;
		data.tweet_message = self.options.tweet.message;
		data.context = self.locker.get_context_data();

		// For developers
		data = cl.hooks.apply_filters( self.locker.id + '.ajax-data', data );
		data = cl.hooks.apply_filters( self.locker.id + '.twitter-tweet.ajax-data', data );

		// Prepare for submit
		data.action = 'mts_cl_connect';
		data.security = __contentlocker.security;

		var request = $.ajax({
			url: __contentlocker.ajaxurl,
			type: 'POST',
			dataType: 'json',
			data: data,

			success: function( response ) {

				if ( ! response || response.error || response.errors ) {
					console.log( 'Unable to tweet:' + request.responseText );
				}

				if ( response && response.error ) {
					self.locker.screen( 'default' );
					self.show_notice( 'Unable to perform the tweet action due to the error: ' + response.error );
					return;
				} else {
					callback();
				}
			},

			error: function() {

				self.locker.screen( 'default' );
				self.show_notice( 'Unable to perform the tweet action due to the unexpected error. See the logs for more details.' );
				console.log( 'Unable to tweet: ' + request.responseText );
			}
		});
	}
});
cl.controls.twitter = TwitterSignin;
