/*!
 * Helper Tools
 *
 * @since 1.0.0
 * @pacakge core
*/

if ( ! String.prototype.includes ) {
	String.prototype.includes = function( search, start ) {

		if ( 'number' !== typeof start ) {
			start = 0;
		}

		if ( start + search.length > this.length ) {
			return false;
		} else {
			return this.indexOf( search, start ) !== -1;
		}
	};
}

var Helpers = {

	extend: function( protoProps, staticProps ) {
		var parent = this;
		var child;

		if ( protoProps && _.has( protoProps, 'constructor' ) ) {
			child = protoProps.constructor;
		} else {
			child = function() {
				return parent.apply( this, arguments );
			};
		}

		_.extend( child, parent, staticProps );

		child.prototype = _.create( parent.prototype, protoProps );
		child.prototype.constructor = child;

		child.__super__ = parent.prototype;

		return child;
	},

	// Hash
	hash: function( str ) {

		if ( ! str || 0 === str.length ) {
			return 0;
		}

		var hash = 5381, i, chr, len;
		for ( i = 0, len = str.length; i < len; i++ ) {
			chr   = str.charCodeAt( i );
			hash  = ( ( hash << 5 ) - hash ) + chr;
			hash &= hash; // Convert to 32bit integer
		}

		hash = hash.toString( 16 );
		hash = hash.replace( '-', '0' );

		return hash;
	},

	// Create GUID
	guid: function() {

		var d = new Date().getTime();
		if ( window.performance && 'function' === typeof window.performance.now ) {
			d += performance.now(); // Use high-precision timer if available
	    }
		var uuid = 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace( /[xy]/g, function( c ) {
			var r = ( d + Math.random() * 16 ) % 16 | 0;
			d = Math.floor( d / 16 );
			return ( 'x' == c ? r : ( r & 0x3 | 0x8 ) ).toString( 16 );
	    });

		return uuid;
	},

	can_blur: {

		cssfilters: (function() {
			var el = document.createElement( 'div' ),
				_browserPrefixes = ' -webkit- -moz- -o- -ms- '.split( ' ' );

			el.style.cssText = _browserPrefixes.join( 'filter' + ':blur(2px); ' );
			return !! el.style.length && ( ( undefined === document.documentMode || document.documentMode > 9 ) );
		})(),

		// https://github.com/Modernizr/Modernizr/blob/master/feature-detects/svg-filters.js
		svgfilters: (function() {
			var result = false;
			try {
				result = undefined !== typeof SVGFEColorMatrixElement  && 2 === SVGFEColorMatrixElement.SVG_FECOLORMATRIX_TYPE_SATURATE;
			} catch ( e ) {

			}
			return result;
		})()
	},

	support_blurring: function() {
		return this.can_blur.cssfilters || this.can_blur.svgfilters;
	},

	is_valid_email: function( val ) {
		var reg = new RegExp( /^((([a-z]|\d|[!#\$%&'\*\+\-\/=\?\^_`{\|}~]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])+(\.([a-z]|\d|[!#\$%&'\*\+\-\/=\?\^_`{\|}~]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])+)*)|((\x22)((((\x20|\x09)*(\x0d\x0a))?(\x20|\x09)+)?(([\x01-\x08\x0b\x0c\x0e-\x1f\x7f]|\x21|[\x23-\x5b]|[\x5d-\x7e]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(\\([\x01-\x09\x0b\x0c\x0d-\x7f]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF]))))*(((\x20|\x09)*(\x0d\x0a))?(\x20|\x09)+)?(\x22)))@((([a-z]|\d|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(([a-z]|\d|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])*([a-z]|\d|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])))\.)+(([a-z]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])|(([a-z]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])([a-z]|\d|-|\.|_|~|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])*([a-z]|[\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF])))\.?$/i );
		return reg.test( val );
	},

	// Store
	store: {
		set: function( key, value, options ) {

			if ( this.is_localstorage() ) {
				return this.local( key, value, options );
			} else {
				return this.cookie( key, value, options );
			}
		},

		get: function( key ) {

			if ( this.is_localstorage() ) {
				return this.local( key );
			} else {
				return this.cookie( key );
			}
		},

		remove: function( key ) {
			if ( ! key ) {
				return false;
			}

			localStorage.removeItem( key );
			this.cookie( key, '', { expires: -1 } );

			return true;
		},

		// LocaStorge
		local: function( key, value, options ) {

			if ( ! key ) {
				return false;
			}

			// Write
			if ( arguments.length > 1 ) {

				value = {
					value: value,
					expires: this.expiry( options.expires )
				};

				localStorage.setItem( key, JSON.stringify( value ) );

				return true;
			}

			// Read
			var item = localStorage.getItem( key );

			if ( ! item ) {
				return false;
			}

			item = JSON.parse( item );

			if ( item.expires && Date.now() > item.expires ) {
				localStorage.removeItem( key );
				return false;
			}

			return item.value;
		},

		// Cookies
		cookie: function( key, value, options ) {

			if ( ! key ) {
				return false;
			}

			// Write
			if ( arguments.length > 1 ) {

				options = $.extend( {}, {
					path: '/',
					expires: '30d'
				}, options );

				options.expires = this.expiry( options.expires );
				var date = new Date();
				date.setTime( options.expires );

				if ( 'object' === typeof value ) {
					value = JSON.stringify( value );
				}

				return ( document.cookie = [
					encodeURIComponent( key ) + '=' + encodeURIComponent( value ),
					options.expires ? '; expires=' + date.toUTCString() : '', // Use expires attribute, max-age is not supported by IE
					options.path    ? '; path=' + options.path : '',
					options.domain  ? '; domain=' + options.domain : '',
					options.secure  ? '; secure' : ''
				].join( '' ) );
			}

			// Read
			var cookies = document.cookie ? document.cookie.split( '; ' ) : [],
				rdecode = /(%[0-9A-Z]{2})+/g,
				i = 0,
				result = false;

			for ( ; i < cookies.length; i++ ) {

				var parts = cookies[i].split( '=' ),
					name = parts[0].replace( rdecode, decodeURIComponent ),
					cookie = parts.slice( 1 ).join( '=' );

				if ( key === name ) {

					if ( '"' === cookie.charAt( 0 ) ) {
						cookie = cookie.slice( 1, -1 );
					}

					try {
						cookie = cookie.replace( rdecode, decodeURIComponent );
						try {
							cookie = JSON.parse( cookie );
						} catch ( e ) {

						}
					}
					catch ( e ) {

					}

					result = cookie;
					break;
				}
			}

			return result;
		},

		is_localstorage: function() {
			return 'undefined' !== typeof Storage;
		},

		expiry: function( val ) {

			if ( ! val ) {
				return false;
			}

			if ( -1 === val ) {
				var d = new Date();
				d.setYear( 1970 );
				return d.getTime();
			}

			var interval = parseInt( val ),
				unit = val.replace( interval, '' );

			if ( 'd' === unit ) {
				interval = interval * 24 * 60 * 60 * 1000;
			}

			if ( 'h' === unit ) {
				interval = interval * 60 * 60 * 1000;
			}

			if ( 'm' === unit ) {
				interval = interval * 60 * 1000;
			}

			if ( 's' === unit ) {
				interval = interval * 1000;
			}

			return Date.now() + interval;
		}
	} // End of Store
};
cl.helpers = Helpers;
