/*!
 * Locker
 *
 * @since 1.0.0
 * @pacakge core
*/
/*jshint -W003*/
var Locker = function( target, opts ) {

	// Variables -------------------------------------------------------
	this.el = target;
	this.options = opts;

	this.controls = {};
	this.container = null;
	this.visitor_id = null;
	this.is_locked = false;
};
Locker.extend = cl.helpers.extend;

_.extend( Locker.prototype, {

	// API ------------------------------------------------------------
	init: function() {

		this.prepare_options();
		this.prepare_group();
		cl.hooks.do_action( 'locker-init', this );
		this.init_hooks();
		this.init_screens();
		this.can_lock();
	},

	// Functions -----------------------------------------------------

	prepare_options: function() {

		// Options
		this.options = $.extend( {}, true, this.defaults, this.options );

		// VisitorID
		this.visitor_id = cl.manager.get_visitor_id();

		// Locker ID
		this.id = this.options.locker_id;

		// Overlap
		if ( this.options.overlap && 'blurring' === this.options.overlap.mode && ! cl.helpers.support_blurring() ) {
			this.options.overlap.mode = cl.settings.get( 'overlap_mode' );
		}
	},

	prepare_group: function() {

		var group = this.options.group,
			self = this;

		_.each( group.buttons, function( button, name ) {

			if ( _.has( cl.controls, name ) ) {
				var control = new cl.controls[name]( button, self );
				self.controls[name] = control;
			}
		});
	},

	init_hooks: function() {

		var self = this;
		self.add_action( 'raw-impress', function() {
			self.do_action( 'impress' );
		});

		// Tracking Stats
		if ( cl.settings.get( 'tracking' ) ) {

			self.add_action( 'unlock', function( sender, senderName ) {

				if ( -1 === $.inArray( sender, [ 'button', 'cross' ] ) ) {
					return;
				}

				if ( 'button' === sender ) {
					self.ping( senderName, 'unlock' );
				} else {
					self.ping( sender, 'skip' );
				}
			});

			var hooks = [ 'impress', 'error', 'got-youtube-subscriber' ];

			$.each( hooks, function( index, value ) {
				self.add_action( value, function() {
					self.ping( value );
				});
			});
		}
	},

	can_lock: function() {

		var self = this,
			what = 'locked';

		// If no button to print
		if ( 0 === _.size( this.options.group.buttons ) ) {
			self.unlock( 'group' );
			return;
		}

		// Check state
		_.each( self.controls, function( control ) {
			var id = self.get_identity() + '_' + control.get_identity(),
				state = self.get_state_storage().request_state( id );

			if ( 'unlocked' === state ) {
				what = 'unlocked';
				self.unlock( 'provider' );
				return;
			}
		});

		if ( 'locked' === what ) {
			self.lock();
		}
	},

	lock: function() {

		if ( this.is_locked ) {
			return;
		}

		var self = this;

		self.render();
		self.render_terms();
		self.render_cross();

		if ( self.options.overlap ) {
			self.overlapBox.fadeIn( 1e3, function() {

				if ( 'scroll' === self.options.overlap.position ) {
					self.overlapBox.pin({
						containerSelector: self.el,
						padding: {
							top: 35,
							bottom: 35
						}
					});
				}
			});
		} else {
			self.container.fadeIn( 1e3 );
		}

		self.is_locked = true;
		self.do_action( 'lock' );
		setTimeout(function() {
			self.track_visibility();
		}, 1500 );
	},

	unlock: function( type, name, control ) {

		type = type || 'api';

		if ( ! this.is_locked ) {
			this.show_content( 'button' === type );
			return false;
		}

		this.show_content( true );
		this.is_locked = false;
		this.do_action( 'unlock', [ type, name, control ] );

		// Change of state
		if ( 'undefined' !== typeof control ) {

			var id = this.get_identity() + '_' + control.get_identity();

			this.get_state_storage().set_state( id, 'unlocked' );
			this.do_action( 'state-changed', [ 'unlocked', type, name, control ] );
		}
	},

	render: function() {

		var self = this,
			content = self.el,
			container = $( $( '#tmpl-locker-' + self.options.locker_id ).html() ),
			_id = 'content-locker-' + self.options.locker_id;

		container.attr( 'id', _id );

		// Make default screen
		container.find( '.mts-cl-wrapper-inner' ).wrapInner( '<div class="mts-cl-screen mts-cl-screen-default"></div>' );
		self._screens['default'] = container.find( '.mts-cl-screen-default' );

		// On later when doing overlap mode
		if ( self.options.overlap ) {

			content.show();
			content.addClass( 'mts-cl-overlap-mode' );

			var overlapBox = self.overlapBox = $( '<div class="mts-cl-overlap-locker-box"></div>' ).hide();
			overlapBox.addClass( 'mts-cl-position-' + self.options.overlap.position );
			overlapBox.append( container );

			var overlap = $( '<div class="mts-cl-overlap-box"></div>' );
			overlap.addClass( 'mts-cl-' + self.options.overlap.mode + '-mode' );
			overlap.addClass( self.options.theme + '-theme' );
			overlap.append( overlapBox );
			overlap.append( '<div class="mts-cl-overlap-background"></div>' );

			if ( 'blurring' === self.options.overlap.mode ) {
				content.wrapInner( '<div class="mts-cl-blur-area"></div>' );
				self.blur_area = content.find( '.mts-cl-blur-area' );
				self.blur_area = content.find( '.mts-cl-blur-area' ).Vague({
					intensity: self.options.overlap.intensity,
					forceSVGUrl: false
				});

				self.blur_area.blur();
			}

			content.append( overlap );
		} else {
			container.hide();
			content.after( container );
		}

		self.container = $( '#' + _id );

		if ( cl.settings.get( 'lazyload' ) ) {
			self.add_action( 'raw-impress', function() {
				self.create_controls();
			});
		} else {
			self.create_controls();
		}
	},

	render_terms: function() {

		if ( ! ( cl.settings.get( 'terms' ) || cl.settings.get( 'policy' ) )  ) {
			return;
		}

		var self = this;

		self.container.addClass( 'mts-cl-has-terms' );
		self.container.find( '.mts-cl-terms .mts-cl-link' ).click(function() {
			var link = $( this ).attr( 'href' );

			window.open( link, 'mts_cl_policies', 'width=550,height=400,resizable=yes,scrollbars=yes' );

			return false;
		});
	},

	render_cross: function() {

		if ( ! this.options.close ) {
			return;
		}

		var self = this;

		$( '<div class="mts-cl-cross" title="' + cl.__.misc.close + '">&times;</div>' ).prependTo( self.container ).on( 'click', function() {
			self.unlock( 'cross', true );
		});
	},

	track_visibility: function() {

		var self = this;

		self.container.appear(function() {

			self.do_action( 'raw-impress' );

		}, { accX: 50, accY: 100 });
	},

	show_content: function( effect ) {

		var self = this;

		if ( self.options.overlap ) {

			if ( self.overlapBox ) {
				self.overlapBox.parent().hide();
			}
			
			if ( self.blur_area ) {
				self.blur_area.unblur();
			}
		} else {
			if ( self.container ) {
				self.container.hide();
			}
		}

		if ( effect ) {
			self.el.fadeIn( 1000, function() {

				if ( self.options.highlight ) {
					self.el.addClass( 'mts-cl-highlight' );

					setTimeout(function() {
						self.el.removeClass( 'mts-cl-highlight' );
					}, 800 );
				}
			});
		} else {
			self.el.show();
		}
	},

	// Stats --------------------------------------------------------
	get_context_data: function() {
		var data = {
			locker_id: this.id,
			post_id: this.options.post_id,
			post_title: document.getElementsByTagName( 'title' )[0].innerHTML,
			post_url: window.location.href
		};

		return data;
	},

	ping: function( eventName, eventType ) {

		var self = this;
		var request = $.ajax({
			url: __contentlocker.ajaxurl,
			type: 'POST',
			dataType: 'json',
			data: {
				action: 'mts_cl_stats',
				security: __contentlocker.security,
				context: self.get_context_data(),
				stats: {
					name: eventName,
					type: eventType
				}
			}
		});

		request.success(function( data ) {

			if ( data && ! data.success ) {
				console.log( data.message );
				return;
			}
		});

		request.error(function( data ) {

			if ( data && ! data.success ) {
				console.log( 'Unexpected error occurred during the ajax request:' );
				console.log( request.responseText );
				return;
			}
		});
	},

	// Utility ------------------------------------------------------

	get_state_storage: function() {

		if ( this._state_storage ) {
			return this._state_storage;
		}

		this._state_storage = new cl.StateStorage( this );

		return this._state_storage;
	},

	get_identity: function() {

		if ( this._identity ) {
			return this._identity;
		}

		this._identity = 'locker_' + this.id;

		return this._identity;
	},

	is_vertical: function() {
		return 'vertical' === this.options.layout;
	},

	has_counters: function() {
		return this.options.group.counters;
	},

	add_action: function( action, callback, priority, seprate ) {
		cl.hooks.add_action( this.id + '.' + action, callback, priority );

		if ( seprate ) {
			cl.hooks.add_action( action, callback, priority );
		}
	},

	do_action: function( action, args, seprate ) {
		args = args || [];
		args.push( this );

		cl.hooks.do_action.apply( undefined, $.merge( [this.id + '.' + action], args ) );
		if ( seprate ) {
			cl.hooks.do_action.apply( undefined, $.merge( [action], args ) );
		}
	},

	// Screen Manager ----------------------------------------------------
	_screens: {},
	_screen_factory: {},

	screen: function( id, options ) {

		var self = this;

		if ( ! self._screen_factory[id] && ! self._screens[id] ) {
			throw new Error( 'The screen with "' + id + '" not found in the locker.' );
		}

		self._current_screen = id;
		self.container.find( '.mts-cl-screen' ).hide();

		if ( self._screens[id] ) {
			self._screens[id].show();
			return;
		}

		var screen = $( '<div class="mts-cl-screen mts-cl-screen' + id + '"></div>' ).hide();
		self.container.find( '.mts-cl-wrapper-inner' ).append( screen );

		self._screen_factory[id]( screen, options );
		self._screens[id] = screen;
		screen.fadeIn( 300 );
	},

	_register_screen: function( id, func ) {
		this._screen_factory[id] = func;
	},

	init_screens: function() {

		var self = this;
		self._current_screen = 'default';

		// Loader Screen
		self._register_screen( 'loader', function( screen ) {

			screen.append( $( '<div class="mts-cl-processing-icon"></div>' ) );
			screen.append( $( '<div class="mts-cl-processing-text">' + cl.__.misc.data_processing + '</div>' ) );
		});

		// Enter Email
		self._register_screen( 'enter-email', function( screen, options ) {

			var header = $( '<div class="mts-cl-header"></div>' );
			screen.append( header );

			if ( options.title ) {
				$( '<div class="mts-cl-header-title">' + options.title + '</div>' ).appendTo( header );
			}
			if ( options.message ) {
				$( '<div class="mts-cl-header-message">' + options.message + '</div>' ).appendTo( header );
			}

			var form = $( '<div class="onp-sl-screen-form"></div>' );
			screen.append( form );

			var email = $( '<div class="mts-cl-field mts-cl-field-email"></div>' ),
				input = $( '<input />', {
					type: 'text',
					name: 'email',
					id: 'mts-cl-input-email',
					placeholder: cl.__.misc.enter_your_email
				} );

			email.append( input );
			form.append( email );

			var submit = $( '<div class="mts-cl-field mts-cl-field-submit"></div>' ),
				button = $( '<button></button>', {
					class: 'mts-cl-button mts-cl-form-button mts-cl-submit mts-cl-button-primary',
					text: options.button_title
				} );

			submit.append( button );
			form.append( submit );

			if ( options.note ) {
				$( '<div class="mts-cl-note mts-cl-nospam"></div>' ).html( options.note ).appendTo( screen );
			}

			screen.find( '.mts-cl-submit' ).click( function() {
				var notice = function( msg ) {

					var notice = $( '<div class="mts-cl-group-notice">' + msg + '</div>' );

					notice.hide();
					screen.find( '.mts-cl-group-error' ).remove();
					screen.append( notice );
					notice.fadeIn( 500 );

					setTimeout(function() {
						notice.fadeOut( 500, function() {
							notice.remove();
						});
					}, 5000 );
				},
				email = $.trim( screen.find( '#mts-cl-input-email' ).val() );

				if ( email && email.length ) {

					if ( cl.helpers.is_valid_email( email ) && options.callback ) {
						options.callback( email );
					} else {
						notice( cl.__.errors.incorrect_email );
					}
				} else {
					notice( cl.__.errors.empty_email );
				}
			});
		});
	}
});
cl.Locker = Locker;
